/* B<Fransch> */ 
#include <stdio.h>
#include <ctype.h>
#include <malloc.h>

#include <unit.h>
#include <city.h>
#include <game.h>
#include <civ-script.h>
#include <city-plan.h>
#include <shared.h>

static int civscr_line_nr;
struct cityplan_lib    CITYPLAN_Lib;

static int civscr_skip_whitespace(FILE *fh);
static int civscr_find_number(FILE* fh, float* number);
static int civscr_find_name(FILE* fh, char  (*name)[MAX_LENGTH_NAME]);
static int civscr_is_spec_op(FILE* fh, const int Op);
static int civscr_find_cityplan(FILE* fh, struct cityplan_lib* pcpl_lib);
static int civscr_find_cityplan_require(FILE * fh, struct cityplan_lib* cpl,
					struct vector_int* change_cp);


int  civscr_skip_whitespace(FILE *fh)
{ int tmp;
  do { 
    tmp = getc(fh); 
    if( tmp == EOF ) return 0;
    if( tmp == '\n' ) civscr_line_nr++;
    if( tmp == ';' ) { /* skip comment */
      while ( (tmp=getc(fh)) != '\n' ) if( tmp==EOF ) return 0;
      ungetc(tmp,fh);
    }
    if( ! isspace(tmp) ) { ungetc(tmp,fh); return 1; }
  } while (1+1==2) ;
}

int civscr_find_number(FILE* fh, float* number)
{ long  pos  = ftell(fh);
  civscr_skip_whitespace(fh);
  if( fscanf(fh,"%f",number) != 1 ) 
    { fseek(fh,pos,SEEK_SET); return 0; }
  return 1;
}

int civscr_find_name(FILE* fh, char  (*name)[MAX_LENGTH_NAME])
{ 
  long  pos = ftell(fh);
  civscr_skip_whitespace(fh); 
  if( fscanf(fh,"%63[-+*a-zA-Z_0-9]s",*name) != 1 ) 
    { fseek(fh,pos,SEEK_SET); return 0; }
  return 1;
}

int civscr_find_conststring(FILE* fh, char  (*name)[MAX_LENGTH_NAME])
{ 
  long  pos;
  civscr_skip_whitespace(fh); 
  pos = ftell(fh);
  if( civscr_is_spec_op(fh,'\"') == 0 ) return 0; /* " (for emacs hilit) */
  if( fscanf(fh,"%63[^\"]s",*name) != 1 )         /* " */
    { fseek(fh,pos,SEEK_SET); return 0; }
  if( civscr_is_spec_op(fh,'\"') == 0 )           /* " */
    { fseek(fh,pos,SEEK_SET); return 0; }
  return 1;
}

int civscr_is_spec_op(FILE* fh, const int Op)
{ int   tmp;
  civscr_skip_whitespace(fh);
  tmp = getc(fh); 
  if( (char)tmp == (char)Op ) return 1;
  ungetc(tmp,fh);
  return 0;
}

int civscr_find_cityplan(FILE* fh, struct cityplan_lib* pcpl_lib)
{
  char              d_name[MAX_LENGTH_NAME];
  char              plan_name[MAX_LENGTH_NAME];
  float             d_number;
  struct cityplan*  city_plan;
  struct build      tmp;

  if( civscr_find_name(fh,&plan_name) == 0 ) {
    fprintf(stderr,"(%i) Error: missing city plan name \n",
	    civscr_line_nr); return 0;
  }
  city_plan = cityplan_lib_get_entry(pcpl_lib,plan_name);
  cityplan_clean(city_plan);

  if( civscr_is_spec_op(fh,'(')==0 ) {
    fprintf(stderr,"(%i) Error: missing '(' after plan name '%s \n",
	    civscr_line_nr, d_name); return 0; 
  }

  do { 

    if( civscr_find_number(fh,&d_number) == 0 ) d_number = 1;
    if( civscr_find_conststring(fh,&d_name) != 0 ) {
      get_build_by_name(d_name,&tmp);
      if( tmp.kind == -1 ) 
	{ fprintf(stderr,"(%i) Error: what means '%s' ?\n",civscr_line_nr,d_name); return 0; }
      /* Append 'd_number' of units to the cityplan */
      for(;(int)d_number>0; d_number--)  cityplan_append(city_plan,tmp);
    }
    else if( civscr_find_name(fh,(char*)d_name) !=0 ) {
      get_build_by_name(d_name,&tmp);
      if( tmp.kind == -1 ) 
	{ fprintf(stderr,"(%i) Error: what means '%s' ?\n",civscr_line_nr,d_name); return 0; }
      for(;(int)d_number>0; d_number--)  cityplan_append(city_plan,tmp);
    }
    else { 
      fprintf(stderr,"(%i) Error: don't know %s\n",civscr_line_nr, tmp); 
      return 0; 
    }

  } while( civscr_is_spec_op(fh,')') == 0 ) ;

  return 1;
}

int civscr_find_cityplan_require(FILE * fh, 
				 struct cityplan_lib* pcpl,
				 struct vector_int* change_cp)
{
  char                  d_name[MAX_LENGTH_NAME];
  char                  plan[MAX_LENGTH_NAME];
  int                   option = 0;                /* cityplan opions */
  struct city*          pcity;
  struct cityplan*      cp;

  if( civscr_find_name(fh,&plan) == 0 ) {
    fprintf(stderr,"(%i) Error: missing city plan name for requirement\n",
	    civscr_line_nr); return 0;
  }

  cp = cityplan_lib_getp_by_name(pcpl,plan); 
  if( cp == 0 ) {
    fprintf(stderr,"(%i) Error: city plan  '%s' does not exist \n",
	    civscr_line_nr, plan); return 0; 
  }
  if( civscr_is_spec_op(fh,'(')==0 ) {
    fprintf(stderr,"(%i) Error: missing '(' after plan name '%s' \n",
	    civscr_line_nr, d_name); return 0; 
  }

  /* read the list of options */
  while( civscr_is_spec_op(fh,')' ) == 0 ) { 
    if( civscr_find_name(fh,&d_name) == 0 ) 
      { fprintf(stderr,"(%i) Error: c \n",civscr_line_nr); return 0; }
    
    if     ( strcmp(d_name,"immediate")==0 )   option |=  CP_OPT_IMMEDIAT;
    else if( strcmp(d_name,"startrandom")==0 ) option |=  CP_OPT_START_RANDOM;
    else if( strcmp(d_name,"random")==0 )      option |=  CP_OPT_TOTAL_RANDOM;
    else { 
      fprintf(stderr,"(%i) warning: unkown option %s ignored \n",civscr_line_nr,d_name);
      return 0; 
    }
  } 

  if( civscr_is_spec_op(fh,'(')==0 ) {
    fprintf(stderr,"(%i) Error: missing '(' after option list of plan '%s' \n",
	    civscr_line_nr, d_name); return 0; 
  }
  /* read the list of cities */
  do { 
    if( civscr_find_conststring(fh,&d_name) == 0 ) 
      { fprintf(stderr,"(%i) Error: city name missing \n",civscr_line_nr); return 0; }
    pcity = game_find_city_by_name(d_name);
    if( pcity == 0 ) 
      { fprintf(stderr,"(%i) Error: city %s not accessible \n",civscr_line_nr,d_name); return 0; }
    pcity->cpi.index = 0;
    pcity->cpi.plan  = cp;
    pcity->cpi.option = option;
    VECTOR_APPEND(int, change_cp, pcity->id,32);
  } while( civscr_is_spec_op(fh,')' ) == 0 ) ;
  return 1;
}

struct vector_int*
/* don't do anything to the return value, if you want to keep the structure copy it */
/* it is a pointer to a static variable, that is managed inside this function       */
/* => no free(..) or so !!!!!!!!!!!!!                                               */
civscr_read_file(const char*             n_planfile, 
		 struct cityplan_lib*    pcpl_lib)
{
  FILE*               fh = fopen(n_planfile,"r");
  char                d_name[MAX_LENGTH_NAME];
  static struct vector_int*  change_cp = 0; /* vector of city ids, where the plan changed                 */
  /*                                   has to be DESTRUCTED from the caller function if return value != 0 */
  /*                                   => VECTOR_DESTRUCT(p) not free !!!!!!!!!!                          */
  if( change_cp == 0  ) {
    change_cp = malloc((size_t)sizeof(struct vector_int));
    VECTOR_CONSTRUCT(int, change_cp,32);
  }
  else 
    VECTOR_CLEAN(change_cp);

  civscr_line_nr = 0;
  
  if( fh==NULL ) 
    { fprintf(stderr,"File not found\n"); VECTOR_DESTRUCT(change_cp); return 0; }

  while( civscr_is_spec_op(fh,'(')==1 ) {
    if( civscr_find_name(fh,&d_name) == 0 ) {
      fprintf(stderr,"(%i) Error: missing function name after '('\n",
	      civscr_line_nr); 
      VECTOR_DESTRUCT(change_cp); return 0;  
    }
    if( strcmp(d_name,"setq") == 0 ) {
      if( civscr_find_cityplan(fh,pcpl_lib) == 0 ) 
	{ VECTOR_DESTRUCT(change_cp); return 0; }
    }
    else if( strcmp(d_name,"require") == 0 ) {
      if( civscr_find_cityplan_require(fh,pcpl_lib,change_cp) == 0 ) 
	{ VECTOR_DESTRUCT(change_cp); return 0; }
    }
    else {
      fprintf(stderr,"(%i) Error: missing function name after '('\n",
	      civscr_line_nr); VECTOR_DESTRUCT(change_cp); return 0; 
    }
    if( civscr_is_spec_op(fh,')' ) == 0 ) {
      fprintf(stderr,"(%i) Error: missing ')'\n",
	      civscr_line_nr); VECTOR_DESTRUCT(change_cp); return 0;  
    }
    civscr_skip_whitespace(fh);
  }
  fprintf(stderr,"(%i) OK\n",civscr_line_nr);
  return change_cp;
}

/* E<Fransch> */

