/* B<Fransch> */ 
#include <game.h>
#include <city-plan.h>
#include <city.h>
#include <unit.h>
#include <malloc.h>
#include <stdio.h>


/* some definitions to make it easier to write function bodies of
**   vector functions append/delete/etc
**   ( remember you have to define a granularity anywhere
**     struct_t_VECTOR_GRANULARITY)
*/

/* please note that this function is only called from cityplan_lib_get_entry() */
/* This should ONLY happen on the clients side !!!                             */
static unsigned  cityplan_lib_new_id(struct cityplan_lib* cpl)
{ return ++(cpl->last_id); }

/************************************************************************
  IMPLEMENTATION cityplan_table								
************************************************************************/
struct cityplan* cityplan_new()
{ 
  struct cityplan* new_cpl = malloc((size_t)(sizeof(struct cityplan)));
  const char     default_name[] = "default-city-plan-name";
  VECTOR_CONSTRUCT(struct build,new_cpl,64); 
  new_cpl->name = (char *)malloc((size_t)(sizeof(char)*strlen(default_name)+1));
  strcpy(new_cpl->name,default_name);
  return new_cpl;
}

void  cityplan_delete(struct cityplan* pcpl)
{  free((void*)(pcpl->name)); VECTOR_DESTRUCT(pcpl); free((void*)(pcpl)); }

void cityplan_copy_plan(struct cityplan* lval, struct cityplan* source)
{ int i;
  lval->end = -1;
  for(i=0; i<source->end; i++) cityplan_append(lval,source->list[i]);
}

struct build*  cityplan_get_pelm(struct cityplan* pcpl, const unsigned Nr)
{ struct build* p_retval; VECTOR_GETP(pcpl,Nr); return p_retval; }   

void  cityplan_append(struct cityplan* cpl, const struct build ToBuild)
{ VECTOR_APPEND(struct build,cpl,ToBuild,32); }

void   cityplan_set_name(struct cityplan* pcpl, const char* Name)
{
  int    NewLength;
  NewLength = strlen(Name);
  if(NewLength > strlen(pcpl->name) ) {
    free(pcpl->name);
    pcpl->name = malloc((size_t)(sizeof(char)*NewLength)+1);                                   
  }
  strcpy(pcpl->name, Name);
}
 
int   DEBUG_cityplan_print(struct cityplan* pcpl)
{
  int i=0, k=0, act_id=0;
  char* name = 0;
  for(i=0; i <= pcpl->end ; i++) {
    act_id = pcpl->list[i].id;
    name = get_name_of_build(&(pcpl->list[i]));
    /* count number of identical build-ids */
    for(k=1; i < pcpl->end && pcpl->list[i+1].id==act_id ; k++) i++;
    fprintf(stderr, "     %3i <%i,%i>   %s\n",
	    k, pcpl->list[i].id, pcpl->list[i].kind, 
	    name);
  }
  return pcpl->end+1;
}

void  cityplan_clean(struct cityplan* pcpl)
{ VECTOR_CLEAN(pcpl); }

/**********************************************************************
 IMPLEMENTATION CITYPLAN-LIBRARY
**********************************************************************/
void   cityplan_lib_construct(struct cityplan_lib* pcpl_lib)
{ VECTOR_CONSTRUCT(struct cityplan*,pcpl_lib,64); 
  pcpl_lib->last_id = 0;
}

struct cityplan*  cityplan_lib_getp(struct cityplan_lib* pcpl_lib, const unsigned Nr)
{ struct cityplan** p_retval; VECTOR_GETP(pcpl_lib,Nr);  return *p_retval; }   


struct cityplan*
cityplan_lib_find_id(struct cityplan_lib* pcpl_lib, const unsigned id)
{
  struct cityplan** it = pcpl_lib->list;
  for(; it <= pcpl_lib->list + pcpl_lib->end; it++) 
    if( (*it)->id == id ) return *it;
  
  return 0;
}

struct cityplan*
cityplan_lib_getp_by_name(struct cityplan_lib* pcpl_lib, const char* name)
{
  int i=0;
  for(i=pcpl_lib->end; i >= 0 ; i--)
    if( strcmp(pcpl_lib->list[i]->name,name)==0 ) return pcpl_lib->list[i];
  
  return 0;
}
 
/* FUNCTION: cityplan_lib_get_entry(..)
   looks if a cityplan with 'name; allready exists and returns a pointer to it.
   if there's no cityplan with that name => create a new one which is directly
   related to this cityplan_lib
*/
struct cityplan*
cityplan_lib_get_entry(struct cityplan_lib* pcpl_lib, const char* name)
{
  int i=0;
  struct cityplan** p_retval;
  struct cityplan*  retval=0;
  int found_nr = pcpl_lib->end + 1; /* choose default found_nr greater then end */
  /*                                       => if not found in loop => no get            */
  for(i=pcpl_lib->end; i >= 0 ; i--)
    if( strcmp(pcpl_lib->list[i]->name,name)==0 ) { found_nr = i; break; }

  if( found_nr != pcpl_lib->end + 1) {
    VECTOR_GETP(pcpl_lib,found_nr); 
  }
  else {
    /* create a new cityplan structure, return pointer to it */
    retval = cityplan_new();
    cityplan_set_name(retval,name);
    retval->id = cityplan_lib_new_id(pcpl_lib);
    VECTOR_APPEND(struct cityplan*, pcpl_lib, retval,32);
    return retval;
  }
  return *p_retval; 
}

void              cityplan_lib_destruct(struct cityplan_lib* pcpl_lib)
{ 
  int i=0;
  /* destruct all plans in the library */
  for(; i < pcpl_lib->end ; i++) VECTOR_DESTRUCT( (pcpl_lib->list[i]) ); 
  /* destruct the library itself */
  VECTOR_DESTRUCT(pcpl_lib); 
}

void  cityplan_lib_insert(struct cityplan_lib* pcpl_lib, 
			  struct cityplan* cpl)
{  
  struct cityplan** found = 0; 
  struct cityplan** it = pcpl_lib->list;
  for(; it <= pcpl_lib->list + pcpl_lib->end; it++)
    if( strcmp((*it)->name,cpl->name)==0 ) { found = it;  break; }

  if( found != 0 ) { 
    /* city-plan with same name found or newer different id => replace */
    free((void*)(*found));
    *found = cpl;
  }
  else {
    VECTOR_APPEND(struct cityplan*,pcpl_lib,cpl,32);
  }
}


int  cityplan_lib_delete(struct cityplan_lib* pcpl_lib, const unsigned Nr)
{ VECTOR_DELETE(pcpl_lib,Nr); }

int  cityplan_lib_delete_by_name(struct cityplan_lib* pcpl_lib, const char* name)
{ int i;
  int delete_nr = pcpl_lib->end + 1; /* choose default delete_nr greater then end */
  /*                                => if not found in loop => no delete          */
  for(i=pcpl_lib->end; i >= 0 ; i--)
    if( strcmp(name,pcpl_lib->list[i]->name)==0 ) { delete_nr = i; break; }
  if( delete_nr == pcpl_lib->end + 1) return 0;
  VECTOR_DESTRUCT(pcpl_lib->list[delete_nr]);  /* destruct the related cityplan */
  VECTOR_DELETE(pcpl_lib,delete_nr);           /* delete the element structure  */
}

int DEBUG_cityplan_lib_print(struct cityplan_lib* pcpl_lib)
{
  int i=0;
  fprintf(stderr,"* cityplans in lib: \n");

  for(i=0; i <= pcpl_lib->end; i++) {
    fprintf(stderr,"  %s (%i/%i) id = %i\n",pcpl_lib->list[i]->name,
	    pcpl_lib->list[i]->end+1, pcpl_lib->list[i]->size,
	    pcpl_lib->list[i]->id);
    DEBUG_cityplan_print(pcpl_lib->list[i]);
  }
  return pcpl_lib->end;
}

void  get_build_by_name(const char* Name, struct build* tmp)
{
  /* is the first character a '*' => special order */
  if(Name[0]=='*') {
    if( strcmp(&(Name[0]),     get_spec_order_name(CP_ORDER_GROWTH) ) == 0 ) 
      { tmp->id = CP_ORDER_GROWTH; tmp->kind = 2; return;  }
    else if( strcmp(&(Name[0]),get_spec_order_name(CP_ORDER_SCIENCE) ) == 0 ) 
      { tmp->id = CP_ORDER_SCIENCE; tmp->kind = 2; return;  }
    else if( strcmp(&(Name[0]),get_spec_order_name(CP_ORDER_DEFENSE) ) == 0 ) 
      { tmp->id = CP_ORDER_DEFENSE; tmp->kind = 2; return;  } 
    else if( strcmp(&(Name[0]),get_spec_order_name(CP_ORDER_SUPPORT) ) == 0 ) 
      { tmp->id = CP_ORDER_SUPPORT; tmp->kind = 2; return;  } 
    else if( strcmp(&(Name[0]),get_spec_order_name(CP_ORDER_MONEY) ) == 0 ) 
      { tmp->id = CP_ORDER_MONEY; tmp->kind = 2; return;  } 
    else if( strcmp(&(Name[0]),get_spec_order_name(CP_ORDER_HAPPY) ) == 0 ) 
      { tmp->id = CP_ORDER_HAPPY; tmp->kind = 2; return;  } 
    else if( strcmp(&(Name[0]),get_spec_order_name(CP_ORDER_SPACESHIP) ) == 0 ) 
      { tmp->id = CP_ORDER_SPACESHIP; tmp->kind = 2; return;  }     
    else if( strcmp(&(Name[0]),get_spec_order_name(CP_ORDER_PRODUCTION) ) == 0 ) 
      { tmp->id = CP_ORDER_PRODUCTION; tmp->kind = 2; return;  } 
    else if( strcmp(&(Name[0]),get_spec_order_name(CP_ORDER_POLUTION) ) == 0 ) 
      { tmp->id = CP_ORDER_POLUTION; tmp->kind = 2; return; } 
    else 
      { tmp->kind = -1; return; }
    
  }

  tmp->id = find_unit_type_by_name((char*)Name);
  if( tmp->id == U_LAST ) {  /* not an unit        */
    tmp->id = find_improvement_by_name((char*)Name);
    if( tmp->id == B_LAST )  /* not an improv.     */
      { tmp->kind = -1; return; }
    tmp->kind = 0;           /* improvment         */
  }
  else {
    tmp->kind = 1;           /* unit               */
  }
}

char*   get_name_of_build(struct build* pbuild)
{
  if(pbuild->kind==2) {
    return get_spec_order_name(pbuild->id);
  }
  if( pbuild->kind==0 )  { /* improvement */
    if( pbuild->id >= B_LAST ) return 0;
    return get_improvement_name(pbuild->id);
  }
  else {                   /* units */
    if( pbuild->id >= U_LAST ) return 0; 
    return unit_name(pbuild->id);
  } 
}

const char* get_spec_order_name(const int SpecOrder)
{
  static const char GROWTH[]  = "*growth";
  static const char SCIENCE[] = "*science";
  static const char DEFENSE[] = "*defense";
  static const char SUPPORT[] = "*support";
  static const char MONEY[]   = "*money";
  static const char HAPPY[]   = "*happy";
  static const char PRODUCTION[] = "*production";
  static const char POLUTION[] =   "*polution";
  static const char SPACESHIP[] =  "*spaceship";
  static const char LAST[] = "\0";

  switch(SpecOrder) { 
  case CP_ORDER_GROWTH: return GROWTH;
  case CP_ORDER_SCIENCE: return SCIENCE;
  case CP_ORDER_DEFENSE: return DEFENSE;
  case CP_ORDER_SUPPORT: return SUPPORT;
  case CP_ORDER_MONEY: return MONEY;
  case CP_ORDER_HAPPY: return HAPPY;
  case CP_ORDER_PRODUCTION: return PRODUCTION;  
  case CP_ORDER_POLUTION: return POLUTION;  
  case CP_ORDER_SPACESHIP: return SPACESHIP;  
  default: return LAST;
  }
/* E<Fransch> */
}
