/* FILE:     city-plan.h
   PURPOSE:  handling of city-plans
   AUTHOR:   Frank R. Schaefer

   - GNU PUBLIC LICENSE - 
*/
/* B<Fransch> */ 

#ifndef  __CITY_PLAN_H__
#define  __CITY_PLAN_H__  
/* struct that defines, what has to be done automatically, when the
   player get's his turn.
*/
#define  VECTOR_BODY(type_t) \
  type_t*           list;    \
  int               end;      /* index of last element in list      */ \
  int               size      /* allocated memory                   */ 


#define VECTOR_CONSTRUCT(type_t,vector,vec_size) {          	       \
  (vector)->list = (type_t*)malloc((size_t)((vec_size)                 \
			       		    * sizeof(type_t)));        \
  (vector)->end = -1;                                                  \
  (vector)->size = (vec_size);     	      			       \
}

#define VECTOR_DESTRUCT(vector) {   \
   free((vector)->list);            \
}

#define VECTOR_CLEAN(vector) {   \
  (vector)->end = -1;            \
}

#define  VECTOR_APPEND(type_t,vector,element,granularity) {                           \
  type_t*  tmp;                                                                       \
  if((vector)->end >= (vector)->size - 1) {                                           \
     tmp = (vector)->list;                                                            \
     (vector)->list = (type_t*)                                                       \
          malloc((size_t)(((vector)->size + (granularity)) * sizeof(type_t)));        \
    memcpy((vector)->list,tmp,(size_t)(sizeof(type_t)*((vector)->end+1)));            \
    free((void*)tmp);                                                                 \
    (vector)->size += (granularity);                                                  \
  }                                                                                   \
  (vector)->list[++((vector)->end)] = (element);                                      \
}

#define  VECTOR_GETP(vector,ElmNr) {                                                  \
if( (ElmNr) > (vector)->end) p_retval=0;  /* element was not in list */               \
else p_retval = &((vector)->list[(ElmNr)]);                                           \
}

#define VECTOR_DELETE(vector,nr)  {                                                   \
  unsigned i;									      \
  if((nr) > (vector)->end) return 0;  /* element was not in vector  */	              \
  /* shift the rest of the vector to cover the deleted element          */            \
  for(i=(nr); i < (vector)->end ; i++) (vector)->list[i]=(vector)->list[i+1];         \
  (vector)->end = (vector)->end > 0 ? (vector)->end - 1 : -1;	                      \
  fprintf(stderr,"deleted  %i\n",(nr));         				      \
  return 1;									      \
}

#define  VECTOR_ITERATE_FW(vector,it)                                                 \
  for((it)=(vector->list); (it) != (vector)->list + (vector)->end + 1; (it)++)

enum cp_spec_orders {  
  /* special orders as contents of a cityplan:
  ** insted of specifiyng a special building, one specifies what it supports
  */
  CP_ORDER_GROWTH,   /* growth => granary, aqu., sew.       */
  CP_ORDER_SCIENCE,  /* science rate => library, university */
  CP_ORDER_DEFENSE,  /* military defense => city-walls,..   */
  CP_ORDER_SUPPORT,  /* military attack => baracks          */
  CP_ORDER_MONEY,    /* money => marketplace, bank          */
  CP_ORDER_HAPPY,    /* happyness => temple, collos. ...    */
  CP_ORDER_PRODUCTION,
  CP_ORDER_POLUTION,
  CP_ORDER_SPACESHIP,
  CP_ORDER_LAST,
};
const char* get_spec_order_name(const int SpecOrder);

struct build {
  int  kind;  /* 0 = improvement 1 = unit, 2 = special order */
  int  id;    /* id of type                                  */
};

struct cityplan {    /* struct for city plans */
  char*        name; /* id s are only distributed by the client and are */
  int          id;   /* only unique for each player                     */
  VECTOR_BODY(struct build);
};

enum cp_option_bits {
  CP_OPT_START_RANDOM = 0x0001,       /* set start index randomly                    */
  CP_OPT_IMMEDIAT     = 0x0002,       /* break up what is actually build             */
  CP_OPT_TOTAL_RANDOM = 0x0004,       /* choose what to build randomly from the list */
};

struct cp_info {
  int                index;     /* act index of thread of control      */
  int                option;    /* options that are set for the cp     */
  /*                               see enum_cp_option above            */
  struct cityplan*   plan;      /* pointer to a city plan              */
};

struct cityplan_lib {                 /* library of cityplans          */
  int       last_id;                  /* last given id in this library */
  VECTOR_BODY(struct cityplan*);
};

void          get_build_by_name(const char*, struct build*);
char*         get_name_of_build(struct build*);

struct cityplan* cityplan_new();
void             cityplan_delete(struct cityplan* pcpl);
void             cityplan_clean(struct cityplan* pcpl);
void             cityplan_copy_plan(struct cityplan* lval, struct cityplan* source);
void             cityplan_append(struct cityplan* cpl, const struct build ToBuild);
void             cityplan_set_name(struct cityplan* pcpl, const char* Name);
int              DEBUG_cityplan_print(struct cityplan* pcpl);

void              cityplan_lib_construct(struct cityplan_lib*);
void              cityplan_lib_destruct(struct cityplan_lib*);
struct cityplan*  cityplan_lib_get_entry(struct cityplan_lib*, const char* name);
struct cityplan*  cityplan_lib_find_id(struct cityplan_lib*, const unsigned int id);
struct cityplan*  cityplan_lib_getp_by_name(struct cityplan_lib*, const char* name);
void              cityplan_lib_insert(struct cityplan_lib*, struct cityplan* pcpl);
int               cityplan_lib_delete_by_name(struct cityplan_lib*, const char* name);
int               DEBUG_cityplan_lib_print(struct cityplan_lib*);

struct vector_int {
  VECTOR_BODY(int);
};

/* Currently unused, records are stored in the city struct directly *_/
** struct cityplan_table {            /_* vector of cityplan-table  *_/
**   VECTOR_BODY(struct cpt_record);  /_* mapping                   *_/
** };                                 /_* city-id <-> cpt_record    *_/
** void                cityplan_table_construct(struct cityplan_table*);
** void                cityplan_table_destruct(struct cityplan_table*);
** struct cpt_record*  cityplan_table_get_precord(struct cityplan_table*, 
** 					       const int CityID);
** struct cpt_record*  cityplan_table_get_entry(struct cityplan_table* pcpt, const int city_id);
** void                cityplan_table_append(struct cityplan_table* cplt, 
** 					  struct cpt_record*  pCityPlanInfo);
** int                 cityplan_table_erase_city(struct cityplan_table* cplt, 
** 					      const int       CityID);
** int                 DEBUG_cityplan_table_print(struct cityplan_table*);
*/

#endif /* __CITY_PLAN_H__ */
/* E<Fransch> */



